#!/usr/bin/perl

package MATT::Passwd;

use strict;
use Exporter;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

@ISA   = qw(Exporter);
@EXPORT = qw(DoesSysUserExist Encrypt BackupMasterPasswd VerifyMasterPasswd InstallGroup );
@EXPORT_OK = qw();
$VERSION = 0.6.1;

use POSIX qw(uname);
use MATT::Utility;

sub DoesSysUserExist
{
	my ($user) = @_;

	if ( $user ne lc $user ) {
		die "\tDoesSysUserExist: FAILED: username must be lower case!\n\n";
	};

	if ( getpwnam($user) && getpwnam($user) > 0 )
	{
		return 1;
	} 
	else { return 0 };
};

sub Encrypt
{
	# crypt the plain text password
	my ($pass, $debug) = @_;

BEGIN {
	eval { require Crypt::PasswdMD5 };
	if ($@) {

		print "\nERROR: Crypt::PasswdMD5 is not installed. Try this:\n\n";

		if ( (uname)[0] eq "FreeBSD" ) {
			print "cd /usr/ports/security/p5-Crypt-PasswdMD5\n";
			print "make install clean\n\n";
		} else {
			print "perl -MCPAN -e shell\n";
			print "install Crypt::PasswdMD5\n\n";
		};
		die "\n";
	} else {
		Crypt::PasswdMD5->import();
	};
};

	my $salt     = rand;
	my $pass_e   = unix_md5_crypt($pass, $salt);

	print "Encrypt: pass_e = $pass_e\n" if $debug;
	return $pass_e;
};

sub BackupMasterPasswd
{
	# Back up master.passwd database
	my $file = "/etc/master.passwd";
   
	SysCmd("sudo cp $file $file.bak");
           
#	use File::Copy;
#	copy($file, "$file.bak")
#		or warn "FATAL: Couldn't back up $file: $!\n";
};

sub VerifyMasterPasswd
{
	# Verify that new master.passwd is larger than prior one
	my ($passwd, $change, $debug) = @_;

	my $new = (stat($passwd))[7];
	my $old = (stat("$passwd.bak"))[7];

	if ( $new == $old ) 
	{
		print "VerifyMasterPasswd: The files are the same size ($new)!\n" if $debug;
	} 
	elsif ( $change eq "grow" && $new > $old )
	{
		print "VerifyMasterPasswd: The file grew ($old to $new) bytes.\n" if $debug;
	} 
	elsif ( $change eq "grow" && $old > $new )
	{
		print "VerifyMasterPasswd: WARNING: The new file is smaller ($new) than $old!\n";
		print "Making archive of master.passwd...";
		ArchiveFile("$passwd.bak");
		print "done.\n";
	} 
	elsif ( $change eq "shrink" && $old > $new )
	{
		if ( ($old - $new) < 300 ) {
			print "VerifyMasterPasswd: The file shrank ($old to $new) bytes.\n" if $debug;
		} else {
			print "VerifyMasterPasswd: WARNING: The file shrank ($old to $new) bytes.\n";
		};
	} else {
		print "VerifyMasterPassword: Yikes, how'd I get here?\n";
	};
};

sub InstallGroup($;$)
{
	my ($group, $gid) = @_;

	my $r = getgrnam($group);

	if ( $r )
	{
		print "InstallGroup: group $group (gid: $r) installed\n";
	}
	else
	{
		print "InstallGroup: installing $group\n";

		my $pw = FindTheBin("pw");

		if ( $gid )
		{
			SysCmd( "$pw groupadd -n $group -g $gid");
		} else {
			SysCmd( "$pw groupadd -n $group");
		};
	};
};

sub InstallUser($;$$)
{
	my ($user, $shell, $homedir) = @_;

	my $r  = getpwnam($user);

	$shell = "/sbin/nologin" if (!$shell);

	if ( getpwnam($user) )
	{
		print "user $user (uid: $r) already exists\n";
	}
	else
	{
		print "installing $user\n";

		my $pw = FindTheBin("pw");

		if (  $user eq "vpopmail" )
		{
			SysCmd("$pw useradd -n $user -u 89 -g 89 -c Vpopmail-Master -d $homedir -s $shell");
		}
		elsif (  $user eq "clamav" )
		{
			my $gid = getgrnam("clamav");
			SysCmd( "$pw useradd -n $user -g $gid -d /nonexistent -c Clam-Antivirus -s $shell");
		}
		else
		{
			if ($homedir)
			{
				SysCmd( "$pw useradd -n $user -d $homedir -s $shell");
			} else {
				SysCmd( "$pw useradd -n $user -s $shell");
			};
		};
	};
};

1;
__END__

=head1 NAME

        MATT::Passwd.pm - Common Unix Passwd functions

=head1 SYNOPSIS

        Passwd.pm is a grouping of frequently used functions I've written

=head1 DESCRIPTION

        Dependencies

        Crypt::PasswdMD5 -  /usr/ports/security/p5-Crypt-PasswdMD5

=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to author.

=head1 TODO

        Don't export any of the symbols by default. Move all symbols
        to EXPORT_OK and explicitely pull in the required ones in 
        programs that need them.

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
