#!/usr/bin/perl
package MATT::Qmail;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA   = qw(Exporter);
@EXPORT = qw(StopQmail StartQmail CheckQmailControl CheckQmailQueue ProcessQmailQueue);
$VERSION = 1.0.1;

use MATT::Utility;

##
# Qmail.pm
##
# -----------------------------------------------------------------------------  

my $qdir     = "/var/qmail/queue";
my $qcontrol = "/service/send";

sub StopQmail
{
	my $svc    = FindTheBin("svc");
	my $svstat = FindTheBin("svstat");

	# send qmail-send a TERM signal
	system "$svc -d $qcontrol";

	# loop up to a thousand seconds waiting for qmail-send to die
	foreach my $i ( 1..1000 ) {
		my $r = `$svstat $qcontrol`;
		chomp $r;
		if ( $r =~ /^.*:\sdown\s[0-9]*\sseconds/ ) {
			print "Yay, we're down!\n";
			return 0;
		} else {
			# if more than 100 seconds passes, lets kill of the qmail-remote
			# processes that are forcing us to wait.
			
			if ($i > 100) {
				system "killall qmail-remote";
			};
			print "$r\n";
		};
		sleep 1;
	};
	return 1;
};

sub StartQmail
{
	my $svc    = FindTheBin("svc");
	my $svstat = FindTheBin("svstat");

	# Start the qmail-send (and related programs)
	system "$svc -u $qcontrol";

	# loop until it's up and running.
	foreach my $i ( 1..100 ) {
		my $r = `$svstat $qcontrol`;
		chomp $r;
		if ( $r =~ /^.*:\sup\s\(pid [0-9]*\)\s[0-9]*\sseconds$/ ) {
			print "Yay, we're up!\n";
			return 0;
		};
		sleep 1;
	};
	return 1;
};

sub CheckQmailControl
{
	my $dir = shift;
	
	if ( ! -d $dir )
	{
		print "HEY! The control directory for qmail-send is not
		in $dir where I expected. Please edit this script
		and set $qcontrol to the appropriate directory!\n";
	};
};

sub CheckQmailQueue
{
	my $dir = shift;
	
	if ( ! -d $dir )
	{
		print "HEY! The queue directory for qmail is not
		$dir where I expect. Please edit this script
		and set $qdir to the appropriate directory!\n";
	};
};

sub ProcessQmailQueue
{
	my $svc    = FindTheBin("svc");

	print "\nSending ALRM signal to qmail-send.\n";
	system "$svc -a $qcontrol";
};

1;
__END__

=head1 NAME

        MATT::Qmail.pm - Common Qmail functions

=head1 SYNOPSIS

        Qmail.pm is a grouping of frequently used functions I've written
        for Qmail.

=head1 DESCRIPTION

=head1 AUTHOR

        Matt Simerson <matt@cadillac.net>

=head1 BUGS

        None known. Report any to author.

=head1 TODO

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
