#!/usr/bin/perl

package MATT::Quota;

##
# Quota.pm by Matt Simerson <matt@cadillac.net>
##

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
require Exporter;

@ISA   = qw(Exporter);
@EXPORT = qw(QuotaSet );
$VERSION = 0.2.0;

use MATT::Utility;
use MATT::FreeBSD;

sub QuotaSet
{
	my ($quotafs, $user, $quota, $debug) = @_;

	my $sudo     = FindTheBin("sudo");
	my $setquota = FindTheBin("setquota");

	if (!-x $setquota)
	{ 
		InstallPort("setquota", "/usr/ports/sysutils/setquota");
		$setquota = FindTheBin("setquota");
	};

	my $quotabump = $quota + 5;
	print "QuotaSet: setting $quotafs quota for user $user to $quota megabytes..." if $debug;
	my $cmd = "$sudo $setquota -u -f $quotafs -bh${quotabump}M -bs${quota}M $user";
	SysCmd($cmd);
	print "done.\n" if $debug;
};

1;
__END__

=head1 NAME

        MATT::Quota.pm - Quota Management functions

=head1 SYNOPSIS

        Quota.pm is a grouping of frequently used functions I've written
        for dealing with user quotas on FreeBSD systems.

=head1 DESCRIPTION

        Dependencies

        setquota   - /usr/ports/sysutils/setquota

=head1 AUTHOR

	Matt Simerson <matt@cadillac.net>

=head1 BUGS

	None known. Report any to matt@cadillac.net.

=head1 TODO

	Needs lots more documentation.
	Support for other platforms (besides FreeBSD)

=head1 SEE ALSO

        http://matt.simerson.net/computing/

=head1 COPYRIGHT

        Copyright 2003, Matt Simerson. All Right Reserved.

=cut
